#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : inn.
# @File         : pooling
# @Time         : 2020/5/20 6:23 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import tensorflow as tf

# 类似np
tf.keras.layers.GlobalMaxPooling1D
tf.keras.layers.GlobalAveragePooling1D

# 步长池化
tf.keras.layers.MaxPooling1D
tf.keras.layers.AveragePooling1D


